#include "config.h"
#include "commands.h"
#include "set.h"
#include "ctype.h"

set_list set_commands[NUMOFCHANSETTINGS+1] =
/*   Command index level user num default description short set0 set1 set2 set3 set4 set5 */
{
  {
    "TOPICFREQ", TOPICFREQ, CO, 1, 4, 1, "How frequently X2 sets the channel topic to its default.",
      "The Topic is refreshed: ",
      {"Never.",
      "Once a day.",
      "Twice a day.",
      "Every 4 hours.",
      "Every hour.",
      ""}
  }
  ,
  {
    "STRICTOP", STRICTOP, MA, 1, 2, 0, "Does X2 allow non-users to be opped by others?",
      "Unknown users can be opped by: ",
      {"Anyone.",
      "Only OPs, MANAGERs, and (CO)OWNERs.",
      "Only MANAGERs and (CO)OWNERs.",
      "", "", ""}
  }
  ,
  {
    "AUTOOP", AUTOOP, MA, 1, 3, 2, "Who does X2 autoop?",
      "X2 will Auto-op: ",
      {"Nobody",
      "Only MANAGERs and (CO)OWNERs.",
      "OPs, MANAGERs, and (CO)OWNERs.",
      "Everyone.",
      "", ""}
  }
  ,
  {
    "PROTECT", PROTECT, CO, 1, 4, 0, "Who does X2 protect from whom?",
      "X2 will protect: ",
      {"No one from anyone.",
      "users from anyone of lower rank.",
      "users from anyone of equal or lower rank.",
      "Only MANAGERs, and (CO)OWNERs from everyone.",
      "Only (CO)OWNERS from everyone.",
      ""}
  }
  ,
  {
    "ENFMODES", ENFMODES, MA, 1, 2, 0, "Should X2 force the default modes?",
      "Default modes can be overridden by: ",
      {"Everyone.",
      "Only OPs, MANAGERs, and (CO)OWNERs.",
      "Only MANAGERs and (CO)OWNERs.",
      "", "", ""}
  }
  ,
  {
      /* This option isn't used anymore, so the name is an empty string */
    "", 5, MA, 1, 1, 0, "Should X2 give ops to passworded users who are not verified?",
      "Users who have set a password, but have NOT authenticated: ",
      {"are NOT opped.",
      "are given ops anyway.",
      "", "", "", ""}
  }
  ,
  {
    "BANTIMEOUT", BANTIMEOUT, MA, 1, 5, 0, "Should X2 remove old bans?",
      "Bans are removed: ",
      {"Never.",
      "After 10 minutes.",
      "After 2 hours.",
      "After 4 hours.",
      "After 24 hours.",
      "After one week."}
  }
  ,
  {
    "VOICE", VOICE, MA, 1, 2, 1, "Should X2 give voice?",
      "Mode +v (voice) given to: ",
      {"Noone.",
      "Peons.",
      "Everyone.",
      "",
      "",
      ""}
  }      
  ,
  {
    "USERINFO", USERINFO, CO, 1, 1, 1, "Show userinfo lines?",
      "Show userinfo lines when users join?: ",
      {"No",
      "Yes",
      "",
      "",
      "",
      ""}
  }
  ,
  {
    "INCHAN", INCHAN, CO, 1, 1, 1, "Does X2 enter the channel?",
     "Should X2 go into the channel?: ",
     {"No",
     "Yes",
     "",
     "",
     "",
     ""}
  }
  ,
  {	/* Option not used anymore */
    "", 10, OW, 0, 1, 0, "Is it suspended?",
     "Has the channel been suspended by CService Admin?: ",
     {"No",
     "Yes",
     "",
     "",
     "",
     ""}
  }
  ,
  {
    "NODELETE", NODELETE, 0, 0, 1, 0, "Is it exempted from deletion?",
     "Exempt from unvisited deletion rules?: ",
    { "No",
     "Yes",
     "",
     "",
     "",
     ""}
  }
  ,
  {
    NULL, 0, 0, 0, 0, 0, NULL,
      "",
      {"", "", "", "", "", ""}
  }
};

struct
{
  char *name;
  int (*function)(UserListType uptr, ChannelListType cptr, char *param);
  int  level;
} on_set_commands[] =
{
  { "url",	    s_url,	    CO },
  { "description",  s_description,  CO },
  { "topic",	    s_topic,	    MA },
  { "email",	    s_email,	    OW },
  { "modes",	    s_mode,	    MA },
  { "mode",	    s_mode,	    MA },
  { NULL,	    null(int(*)()), 0  }
};

struct
{
  char *name;
  int (*function)(UserListType uptr, char *param);
} on_xset_commands[] =
{
  { "pingoutconnect", xs_pingoutconnect },
  { "autorouting",    xs_autorouting    },
  { "autoconnect",    xs_autoconnect    },
  { "gatecheck",      xs_gatecheck      },
  { "sub7scan",       xs_sub7scan       },
  { "maxclones",      xs_maxclones      },
  { "cloneres",	      xs_cloneres       },
  { "debug",	      xs_debug          },
  { "dbtarget",	      xs_dbtarget       },
  { NULL,	      null(int(*)())    }
};

void ShowSet(UserListType uptr, ChannelListType cptr, HandleListType hptr)
{
    int i;
    char ModeString[LINEBUFFSIZE];

    send_to_user(uptr, "=----------- %s Settings -------------=", cptr->Name);
    send_to_user(uptr, "      \002Topic \002: %s", cptr->Topic);
    send_to_user(uptr, "      \002Modes \002: %s", GetModeString(ModeString, cptr->PlusModes, cptr->MinModes, cptr->Key, cptr->Limit));
    send_to_user(uptr, "        \002URL \002: %s", cptr->URL);

    if (IsChanOwner(hptr))
    {
        send_to_user(uptr, "     \002E-MAIL \002: %s", cptr->Email);
        send_to_user(uptr, "NOTE: E-mail is visible to IRC Ops and Owners only;");
        send_to_user(uptr, "It is used for verification of identity and contacting you.");
    }
    else if (IsOper(uptr))
    {
        send_to_user(uptr, "     \002E-MAIL \002: %s", cptr->Email);
        send_to_user(uptr, "NOTE: E-mail is for official IRC Op use ONLY!  Do not give it out!");
    }

    send_to_user(uptr, "\002Description \002: %s", cptr->Description);
    for (i = 0; set_commands[i].name != NULL; i++)
    {
        if (set_commands[i].name[0])
        {
            if(set_commands[i].user)  /* If its not an IRCOP only command */
                send_to_user(uptr, "\002%10s\002 : %d  \002%s\002%s", set_commands[i].name,
                             cptr->settings[set_commands[i].index], set_commands[i].pre,
	                     set_commands[i].settings[cptr->settings[set_commands[i].index]]);
	}
    }
    send_to_user(uptr, "=- The following are set by IRCOPS only -=");
    for (i = 0; set_commands[i].name != NULL; i++)
    {
        if (set_commands[i].name[0])
        {
            if(!set_commands[i].user)  /* If it IS an IRCOP only command */
                send_to_user(uptr, "\002%10s\002 : %d  \002%s\002%s", set_commands[i].name,
                             cptr->settings[set_commands[i].index], set_commands[i].pre,
                             set_commands[i].settings[cptr->settings[set_commands[i].index]]);
        }
    }
    send_to_user(uptr, "=----------------- End ------------------=");
}

int s_url(UserListType uptr, ChannelListType cptr, char *param)
{
    if(!param)
        send_to_user(uptr, "\002URL\002: %s", cptr->URL);
    else
    {
        free(cptr->URL);
        if(strlen(param) > MAXURLLEN)
            param[MAXURLLEN] = '\0';
        if((cptr->URL = (char *) malloc(strlen(param) + 1)))
	{
            strcpy(cptr->URL, param);
            send_to_user(uptr, "\002URL set to\002: %s", cptr->URL);
	}
        else
	{
            send_to_user(uptr, "\002Error allocating memory for new URL!\002");
            return(FALSE); 
	}
        WriteChanOpFile(cptr);
    }
    return(TRUE);
}

int s_description(UserListType uptr, ChannelListType cptr, char *param)
{
    if(!param)
        send_to_user(uptr, "\002Description\002: %s", cptr->Description);
    else
    {
        free(cptr->Description);
        if(strlen(param) > MAXDESCLEN)
            param[MAXDESCLEN] = '\0';
        if((cptr->Description = (char *) malloc(strlen(param) + 1)))
	{
            strcpy(cptr->Description, param);
            send_to_user(uptr, "\002Description set to\002: %s", cptr->Description);
	}
        else
	{
            send_to_user(uptr, "\002Error allocating memory for new description!\002");
            return(FALSE);
	}
        WriteChanOpFile(cptr);
    }
    return(TRUE);
}

int s_topic(UserListType uptr, ChannelListType cptr, char *param)
{
    if (!param)
        send_to_user(uptr, "\002Topic\002: %s", cptr->Topic);
    else
    {
        free(cptr->Topic);
        if(strlen(param) > MAXTOPICLEN)
            param[MAXTOPICLEN] = '\0';
        if((cptr->Topic = (char *) malloc(strlen(param) + 1)))
	{
	    strcpy(cptr->Topic, param);
     	    send_to_user(uptr, "\002Default topic set to\002: %s", cptr->Topic);
	}
        else
	{
	    send_to_user(uptr, "\002Error allocating memory for new topic!");
   	    return(FALSE);
	}
        WriteChanOpFile(cptr);
    }
    return(TRUE);
}

int s_email(UserListType uptr, ChannelListType cptr, char *param)
{
    if (!param)
    {
        send_to_user(uptr, "\002E-mail\002: %s [visible to owner and IRC Operators only]", cptr->Email);
    }
    else
    {
        free(cptr->Email);
        if(strlen(param) > MAXEMAILLEN)
            param[MAXEMAILLEN] = '\0';
        if((cptr->Email = (char *) malloc(strlen(param) + 10)))
	{
            strcpy(cptr->Email, param);
            send_to_user(uptr, "\002Owner e-mail set to\002: %s", cptr->Email);
	}
        else
	{
            send_to_user(uptr, "\002Error allocating memory for new e-mail!");
            return(FALSE);
	}
        WriteChanOpFile(cptr);
    }
    return(TRUE);
}

int s_mode(UserListType uptr, ChannelListType cptr, char *param)
{
    int n;
    char ModeString[LINEBUFFSIZE];

    if (!param)
    {
        send_to_user(uptr, "\002Mode: %s",
		     GetModeString(ModeString, cptr->PlusModes,
			           cptr->MinModes, cptr->Key, cptr->Limit));
    }
    else
    {
        for(n=strlen(param)-1;n>=0;n--)
	{
            if(isspace(param[n]))
                param[n] = '\0';
            else 
                break;
	}
        ConvertModeString(param, &cptr->PlusModes, &cptr->MinModes, &cptr->Key, &cptr->Limit);
        if(cptr->settings[INCHAN])
	{
            send_chanmode(cptr, "-k %s", cptr->CurrentKey);
            send_chanmode(cptr, "%s", GetModeString(ModeString, cptr->PlusModes, cptr->MinModes,
			  cptr->Key, cptr->Limit));
	}
        send_to_user(uptr, "\002Default mode set to: %s",
		     ModeString);
        if (cptr->Key)
	    safecopy(cptr->CurrentKey, cptr->Key, MAXCHANKEY);
        else
	    strcpy(cptr->CurrentKey, "");

        WriteChanOpFile(cptr);
    }
    return(TRUE);
}

void ShowSetOption(UserListType uptr, ChannelListType cptr, int i)
{
  int curset;
  int j;

  curset = cptr->settings[set_commands[i].index];
  send_to_user(uptr, "=-----------------------------=");
  send_to_user(uptr, "\002%s\002: %s\002", set_commands[i].name, set_commands[i].pre);
  send_to_user(uptr, "=---- \002*\002 indicates current ----=");
  for (j = 0; j < 6; j++)
  {
    if (set_commands[i].settings[j][0] == 0)
      break;
    if (j == curset)
      send_to_user(uptr, "  %d: [\002*\002] %s", j, set_commands[i].settings[j]);
    else
      send_to_user(uptr, "  %d: [ ] %s", j, set_commands[i].settings[j]);
  }
  send_to_user(uptr, "=------------ End ------------=");
}

int do_set(C_PARAM)
{
  char ModeString[LINEBUFFSIZE];
  char *option, *param;
  int i, j, value;

  option = strtok(rest, " ");
  param = strtok(NULL, "\n");

  if (!option)
  {
    ShowSet(uptr, cptr, hptr);
    return(TRUE);
  }

  for (i = 0; on_set_commands[i].name; ++i)
  {
    if (!strcasecmp(option, on_set_commands[i].name))
    {
      if (IsGodOper(uptr) || (hptr && (userlevelbychar(hptr->Level) >= on_set_commands[i].level)))
	return on_set_commands[i].function(uptr, cptr, param);
      else 
      {
	send_to_user(uptr, "\002set %s\002: Access denied.", on_set_commands[i].name);
	return FALSE;
      }
    }
  }

  for (i = 0; set_commands[i].name != NULL; i++)
  {
    if (!set_commands[i].name[0])
      continue;

    if (!strcasecmp(option, set_commands[i].name))
    {
      if (!param)		/* No parameter. */
      {
	ShowSetOption(uptr, cptr, i);
	return(TRUE);
      }
      else if (!IsGodOper(uptr) && (!hptr || userlevelbychar(hptr->Level) < set_commands[i].level))
      {
	send_to_user(uptr, "\002set %s\002: Access denied.", param);
	return(FALSE);
      }
      if (!set_commands[i].user && !IsGodOper(uptr))
      {
	send_to_user(uptr, "\002%s\002 can only be modified by the Channel Service administrators.", param);
	return(FALSE);
      }
      if (sscanf(param, "%d", &value) != 1)
      {
        send_to_user(uptr, "\002Error: Parameter must be a number.\002");
        return(FALSE);
      }
      if ((value < 0) || (value > set_commands[i].num))
      {
	send_to_user(uptr, "\002Error: %s must be in the range 0-%d.", set_commands[i].name,
		     set_commands[i].num);
	return(FALSE);
      }

      if(value == cptr->settings[set_commands[i].index])
      {
	  send_to_user(uptr, "%s already set to %d", option, value);
	  return(FALSE);
      }
      if(set_commands[i].index == INCHAN && cptr->Suspended)
      {
	send_to_user(uptr, "Channel %s has been suspended by the Channel Service Admins.", cptr->Name);
	send_to_user(uptr, "E-mail cservice@afternet.org for more information.");
        return(FALSE);
      }

      /* everything is ok, so apply the new value */
      cptr->settings[set_commands[i].index] = value;                                             
      WriteChanOpFile(cptr);                                                            
      send_to_user(uptr, "\002%s\002 set to %d. \"\002%s\002%s\"", set_commands[i].name,         
		  value, set_commands[i].pre, set_commands[i].settings[value]);                        

      /* check if inchan changed */
      if(set_commands[i].index == INCHAN)
      {
	if(cptr->settings[INCHAN] == 0)  /* Leave the channel */
	{
	  send_to_user(uptr, "Leaving disabled channel %s.  \002/msg %s %s set INCHAN 1\002 to re-enable.",
	               cptr->Name, IrcNick, cptr->Name);
	  send_part(cptr, "Disabled by %s.", uptr->Nick);
	}
	else  /* Join the channel */
	{
	  send_to_user(uptr, "Rejoining channel %s.", cptr->Name);
	  send_join(cptr);
	  send_opchan(cptr);
	  send_chanmode(cptr, "%s", GetModeString(ModeString, cptr->PlusModes, 
			cptr->MinModes, cptr->Key, cptr->Limit));

	  if (strcmp(cptr->Topic, "*") && cptr->settings[TOPICFREQ])
	    send_topic(cptr, cptr->Topic);
	}
      }
      return(TRUE);
    }
  }
  send_to_user(uptr, "Unknown setting [\002%s\002].", option);
  return(FALSE);
}


void ShowXSet(UserListType uptr)
{
  send_to_user(uptr, "=------- X2 Net Settings -------------=");
  send_to_user(uptr, "\002  PINGOUTCONNECT\002: %s", GetSetting(PINGOUTCONNECT));
  send_to_user(uptr, "\002  AUTOROUTING   \002: %s", GetSetting(AUTOROUTING));
  send_to_user(uptr, "\002  AUTOCONNECT   \002: %s", GetSetting(AUTOCONNECT));
  send_to_user(uptr, "\002  GATECHECK     \002: %s", GetWinGateMet(GATECHECK));
  send_to_user(uptr, "\002  MAXCLONES     \002: %d", MAXCLONES);
  send_to_user(uptr, "\002  CLONERES      \002: %s", GetCloneResMet(CLONERES));
  send_to_user(uptr, "\002  SUB7SCAN      \002: %s", GetWinGateMet(SUB7SCAN));
  send_to_user(uptr, "\002  DEBUG         \002: %d", DEBUG);
  if (*DBTARGET)
    send_to_user(uptr, "\002  DBTARGET      \002: %s", DBTARGET);
  else
    send_to_user(uptr, "\002  DBTARGET      \002: NULL");
  send_to_user(uptr, "=----------------- End ------------------=");
}

int GetVal(UserListType uptr, char *param, int Max, int *Val)
{
  *Val = atoi(param);
  if ((*Val < 0) || (*Val > Max))
  {
    send_to_user(uptr, "\002Error:\002 Value must be in the range 0..%d.", Max);
    return(FALSE);
  }          
  return(TRUE);
}

int xs_pingoutconnect(UserListType uptr, char *param)
{
  int Val;

  if (!GetVal(uptr, param, 1, &Val))
  {
    return(FALSE);
  }
  if (PINGOUTCONNECT == Val)
  {
    send_to_user(uptr, "PINGOUTCONNECT is already set to %d.", PINGOUTCONNECT);
    return(FALSE);
  }
  PINGOUTCONNECT = Val;
  send_wallops("PINGOUTCONNECT set to %s by %s.", GetSetting(PINGOUTCONNECT), uptr); 
}

int xs_autorouting(UserListType uptr, char *param)
{
  int Val;

  if (!GetVal(uptr, param, 1, &Val))
  {
    return(FALSE);
  }
  if (AUTOROUTING == Val)
  {
    send_to_user(uptr, "AUTOROUTING is already set to %d.", AUTOROUTING);
    return(FALSE);
  }
  AUTOROUTING = Val;
  send_wallops("AUTOROUTING set to %s by %s.", GetSetting(AUTOROUTING), uptr->Nick);
}

int xs_autoconnect(UserListType uptr, char *param)
{
  int Val;

  if (!GetVal(uptr, param, 1, &Val))
  {
    return(FALSE);
  }
  if (AUTOCONNECT == Val)
  {
    send_to_user(uptr, "AUTOCONNECT is already set to %d.", AUTOCONNECT);
    return(FALSE);
  }
  AUTOCONNECT = Val;
  send_wallops("AUTOCONNECT %s by %s.", GetSetting(AUTOCONNECT), uptr->Nick);
}

int xs_sub7scan(UserListType uptr, char *param)
{
  int Val;

  if (!GetVal(uptr, param, 2, &Val))
  {
    return(FALSE);
  }
  if (SUB7SCAN == Val)
  {
    send_to_user(uptr, "SUB7SCAN is already set to %d.", SUB7SCAN);
    return(FALSE);
  }
  SUB7SCAN = Val;
  send_wallops("SUB7SCAN set to %d by %s.", SUB7SCAN, uptr->Nick);
}

int xs_gatecheck(UserListType uptr, char *param)
{
  int Val;

  if (!GetVal(uptr, param, 2, &Val))
  {
    return(FALSE);
  }
  if (GATECHECK == Val)
  {
    send_to_user(uptr, "GATECHECK is already set to %d.", GATECHECK);
    return(FALSE);
  }
  GATECHECK = Val;
  send_wallops("GATECHECK set to %d by %s.", GATECHECK, uptr->Nick);
}

int xs_maxclones(UserListType uptr, char *param)
{
  int Val;

  if (!GetVal(uptr, param, 99, &Val))
  {
    return(FALSE);
  }
  if (MAXCLONES == Val)
  {
    send_to_user(uptr, "MAXCLONES is already set to %d", MAXCLONES);
    return(FALSE);
  }
  MAXCLONES = Val;
  send_wallops("MAXCLONES set to %d by %s.", MAXCLONES, uptr->Nick);
}

int xs_cloneres(UserListType uptr, char *param)
{
  int Val;

  if (!GetVal(uptr, param, 3, &Val))
  {
    return(FALSE);
  }
  if (CLONERES == Val)
  {
    send_to_user(uptr, "CLONERES is already set to %d.", CLONERES);
    return(FALSE);
  }
  CLONERES = Val;
  send_wallops("CLONERES set to %s by %s.", GetCloneResMet(CLONERES), uptr->Nick);
}

int xs_debug(UserListType uptr, char *param)
{
  int Val;

  if (!GetVal(uptr, param, 99, &Val))
  {
    return(FALSE);
  }
  DEBUG = Val;
  send_wallops("DEBUG set to %d by %s.", DEBUG, uptr->Nick);
  return(TRUE);
}

int xs_dbtarget(UserListType uptr, char *param)
{
  if (param && *param && (strcasecmp("NULL", param)))
  {
    safecopy(DBTARGET, param, MAXNICKLEN);
    send_wallops("DEBUG TARGET set to %s by %s.", DBTARGET, uptr->Nick);
    return(TRUE);
  }
  else
  {
    strcpy(DBTARGET, "");
    send_wallops("Debugging disabled by %s.", uptr->Nick);
    return(TRUE);
  }
}

int do_xset(C_PARAM)
{
  char *Option, *param;
  int Val;
  int i;

  Option = strtok(rest, " ");
  param = strtok(NULL, "\0");

  if (!Option)
  {
    ShowXSet(uptr);
    return(TRUE);
  }

  if (!IsOper(uptr))
  {
    send_to_user(uptr, "\002xset %s\002: Access denied.", Option);
    return(FALSE);
  }

  if (!param)
  {
    send_to_user(uptr, "\002Set %s to what?\002", Option);
    if (!strcasecmp(Option, "ENFORCE"))
      send_to_user(uptr, "\0020 = disabled, 1 = enabled.\002");
    if (!strcasecmp(Option, "GATECHECK")) 
      send_to_user(uptr, "\0020 = no action, 1 = warn IRC Ops, 2 = GLINE.\002");
    if (!strcasecmp(Option, "CLONERES"))
      send_to_user(uptr, "\0020 = no action, 1 = warn IRC Ops, 2 = Kill, 3 = Gline\002");
    if (!strcasecmp(Option, "SUB7SCAN"))
      send_to_user(uptr, "\0020 = no action, 1 = warn IRC Ops, 2 = Gline.\002");
    return(FALSE);
  }

  for (i = 0; on_xset_commands[i].name; ++i)
  {
    if (!strcasecmp(on_xset_commands[i].name, Option))
    {
      return(on_xset_commands[i].function(uptr, param));
    }
  }

  send_to_user(uptr, "Unknown setting '%s'.", Option);
  return(FALSE);
}
